#!/bin/bash

#set default values to be used when not passed as command line options 

# the default value used when --loopCount is not specified
defaultLoopCount=3

# the default value used when --timeLapse is not specified
defaultTimeLapse=0.40

#########################################
##### DO NOT MODIFY BELOW THIS LINE #####
#########################################

die () {
 echo FATAL ERROR: $@ >&2
 exit 1;
}
#set the PATH to include /usr/local/bin
if test "$PATH" = "${PATH//\/usr\/local\/bin}"; then
  export PATH=/usr/local/bin:$PATH
fi
declare -i squareOutput=0

# parse arguments for recognized flags
while (( $# > 0 )) ; do
 echo $1 = $2
 case "$1" in
    -i|--inputFile)
    inputFile="$2"
    shift # past argument
    ;;
    -o|--outputFile)
    outputFile="$2"
    shift # past argument
    ;;
    -v|--overlayFile)
    overlayFile="$2"
    shift # past argument
    ;;
    -l|--loopCount)
    loopCount="$2"
    shift # past argument
    ;;
    -t|--timeLapse)
    timeLapse="$2"
    shift # past argument
    ;;
    -q|--square)
    squareOutput=1
    ;;
    *)
    # unknown option
    echo unrecognized option $1
    break
    ;;
 esac
 shift # past argument or value
done

#check for fixed arguments usage
if test -z "${inputFile}" && (($# == 5)); then
 echo using fixed arguments
 #arg 1 is path to input file
 inputFile="${1}"

 #arg 2 is path to input file
 outputFile="${2}"

 #arg 3 is overlay file path
 overlayFile="${3}"

 #arg 4 is loop count (a loop is forward and reverse)
 loopCount="${4:-3}"

 #arg 5 is pts (timeLapse)
 timeLapse="${5:-0.4}"

fi

#error check the input arguments

# verify input file
test -r "${inputFile}" || die unreadable input file "${inputFile}"
#verify output path
test "${outputFile}" || die you must specify an output file path
#verify overlay file type
if test "${overlayFile}"; then
 overlayFileType=$(file  "${overlayFile}")
 test "${overlayFileType}" != "${overlayFileType//PNG image data}" || die overlay files must be PNG type
fi
#validate loopCount
if test -z "${loopCount}" ;then
 loopCount=$defaultLoopCount
 echo "--loopCount = $loopCount (using default value)"
fi
#validate timeLapse
if test -z "${timeLapse}" ;then
 timeLapse=$defaultTimeLapse
 echo "--timeLapse = $timeLapse (using default value)"
fi
#0. may need to encode to specs first

inputFileBase="${inputFile//.m??}"

#set file extension automatically
fileExt="${inputFile//${inputFileBase}}"

#1. remove audio (and downconvert video)
workFile1="${inputFileBase}.noAudio${fileExt}"
ffmpeg -nostats -loglevel error -i "${inputFile}" -pix_fmt yuv420p -preset superfast -r 30 -crf 28 -s 1280x720 -c:v libx264 -an -y "${workFile1}"
#ffmpeg -nostats -loglevel error -i "${inputFile}" -vcodec copy -an -y "${workFile1}"

#2. time lapse
workFile2="${inputFileBase}.timeLapse${fileExt}"
ffmpeg -nostats -loglevel error -i "${workFile1}" -filter_complex "[0:v]setpts=${timeLapse:-0.25}*PTS[v]" -map "[v]" -y "${workFile2}"

#3. make it square
if((squareOutput)); then
  echo converting output to square crop
  workFile3="${inputFileBase}.square${fileExt}"
  ffmpeg -nostats -loglevel error -i "${workFile2}" -filter:v "crop=720:720:280:0" -y "${workFile3}"
else
  workFile3="${workFile2}"
fi

#4. overlay
if test -r "${overlayFile}"; then
workFile4="${inputFileBase}.overlay${fileExt}"
ffmpeg -nostats -loglevel error -i "${workFile3}" -i "${overlayFile}" -filter_complex "[0:v]overlay=0:0[v]" -map "[v]" -y "${workFile4}"
else
workFile4="${workFile3}"
fi


#5.reverse
workFile5="${inputFileBase}.reverse${fileExt}"
ffmpeg -nostats -loglevel error -i "${workFile4}" -filter_complex "[0:v]reverse[v]" -map "[v]" -y "${workFile5}"

#6. concat it n-times
#fwd1-rev1-fwd2-rev2-fwd3-rev3…fwdN-revN
workFile6="${inputFileBase}.looper${fileExt}"
if((loopCount > 0)); then
  ffmpeg -nostats -loglevel error -f concat -safe 0 -i <(for (( ((i=0)) ; ((i<loopCount)) ; ((i++)) )) ; do printf "file %s\n" "${workFile4}" "${workFile5}" ; done) -c copy -y "${workFile6}"
else
  workFile6="${workFile4}"
fi

# move product to output path
cp "${workFile6}" "${outputFile}"

rm -rf "${workFile1}"
rm -rf "${workFile2}"
rm -rf "${workFile3}"
rm -rf "${workFile4}"
rm -rf "${workFile5}"
rm -rf "${workFile6}"
rm -rf "${workFile7}"
